import { syntaxTree } from "@codemirror/language";
import { StateField } from "@codemirror/state";
import {
  Decoration,
  EditorView,
  WidgetType
} from "@codemirror/view";
import DOMPurify from "dompurify";
import { isCursorInRange } from "../util";
function extractHTMLBlocks(state) {
  const blocks = new Array();
  syntaxTree(state).iterate({
    enter({ from, to, name }) {
      if (name !== "HTMLBlock")
        return;
      if (isCursorInRange(state, [from, to]))
        return;
      const html = state.sliceDoc(from, to);
      const content = DOMPurify.sanitize(html, {
        FORBID_ATTR: ["style"]
      });
      blocks.push({
        from,
        to,
        content
      });
    }
  });
  return blocks;
}
function blockToDecoration(blocks) {
  return blocks.map(
    (block) => Decoration.widget({
      widget: new HTMLBlockWidget(block),
      block: true,
      side: 1
    }).range(block.to)
  );
}
const htmlBlock = StateField.define({
  create(state) {
    return Decoration.set(blockToDecoration(extractHTMLBlocks(state)));
  },
  update(value, tx) {
    if (tx.docChanged || tx.selection) {
      return Decoration.set(
        blockToDecoration(extractHTMLBlocks(tx.state))
      );
    }
    return value.map(tx.changes);
  },
  provide(field) {
    return EditorView.decorations.from(field);
  }
});
class HTMLBlockWidget extends WidgetType {
  constructor(data, isInline) {
    super();
    this.data = data;
    this.isInline = isInline;
  }
  toDOM() {
    const dom = document.createElement("div");
    dom.style.display = this.isInline ? "inline" : "block";
    dom.style.overflow = "auto";
    dom.innerHTML = this.data.content;
    return dom;
  }
  eq(widget) {
    return JSON.stringify(widget.data) === JSON.stringify(this.data);
  }
}
export {
  htmlBlock
};
//# sourceMappingURL=html.js.map
