var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
import { syntaxTree } from "@codemirror/language";
import {
  StateField,
  StateEffect
} from "@codemirror/state";
import {
  Decoration,
  WidgetType,
  EditorView
} from "@codemirror/view";
import { image as classes } from "../classes";
var WidgetState = /* @__PURE__ */ ((WidgetState2) => {
  WidgetState2[WidgetState2["INITIAL"] = 0] = "INITIAL";
  WidgetState2[WidgetState2["LOADED"] = 1] = "LOADED";
  return WidgetState2;
})(WidgetState || {});
const imageLoadedEffect = StateEffect.define();
const imagePreview = StateField.define({
  create(state) {
    const images = extractImages(state);
    const decorations = images.map(
      (img) => (
        // This does not need to be a block widget
        Decoration.widget({
          widget: new ImagePreviewWidget(img, 0 /* INITIAL */),
          info: img,
          src: img.src
        }).range(img.to)
      )
    );
    return Decoration.set(decorations, true);
  },
  update(value, tx) {
    const loadedImages = tx.effects.filter(
      (effect) => effect.is(imageLoadedEffect)
    );
    if (tx.docChanged || loadedImages.length > 0) {
      const images = extractImages(tx.state);
      const previous = value.iter();
      const previousSpecs = new Array();
      while (previous.value !== null) {
        previousSpecs.push(previous.value.spec.info);
        previous.next();
      }
      const decorations = images.map((img) => {
        var _a;
        const hasImageLoaded = Boolean(
          loadedImages.find(
            (effect) => effect.value.src === img.src
          ) || ((_a = previousSpecs.find((spec) => spec.src === img.src)) == null ? void 0 : _a.loaded)
        );
        return Decoration.widget({
          widget: new ImagePreviewWidget(
            img,
            hasImageLoaded ? 1 /* LOADED */ : 0 /* INITIAL */
          ),
          // Create returns a inline widget, return inline image
          // if image is not loaded for consistency.
          block: hasImageLoaded ? true : false,
          src: img.src,
          side: 1,
          // This is important to keep track of loaded images
          info: __spreadProps(__spreadValues({}, img), { loaded: hasImageLoaded })
        }).range(img.to);
      });
      return Decoration.set(decorations, true);
    }
    return value.map(tx.changes);
  },
  provide(field) {
    return EditorView.decorations.from(field);
  }
});
const imageTextRE = /(?:!\[)(.*?)(?:\])/;
function extractImages(state) {
  const imageUrls = [];
  syntaxTree(state).iterate({
    enter: ({ name, node, from, to }) => {
      if (name !== "Image")
        return;
      const alt = state.sliceDoc(from, to).match(imageTextRE).pop();
      const urlNode = node.getChild("URL");
      if (urlNode) {
        const url = state.sliceDoc(urlNode.from, urlNode.to);
        imageUrls.push({ src: url, from, to, alt });
      }
    }
  });
  return imageUrls;
}
class ImagePreviewWidget extends WidgetType {
  constructor(info, state) {
    super();
    this.info = info;
    this.state = state;
  }
  toDOM(view) {
    const img = new Image();
    img.classList.add(classes.widget);
    img.src = this.info.src;
    img.alt = this.info.alt;
    img.addEventListener("load", () => {
      const tx = {};
      if (this.state === 0 /* INITIAL */) {
        tx.effects = [
          // Indicate image has loaded by setting the loaded value
          imageLoadedEffect.of(__spreadProps(__spreadValues({}, this.info), { loaded: true }))
        ];
      }
      view.dispatch(tx);
    });
    if (this.state === 1 /* LOADED */)
      return img;
    else
      return new Image();
  }
  eq(widget) {
    return JSON.stringify(widget.info) === JSON.stringify(this.info) && widget.state === this.state;
  }
}
export {
  WidgetState,
  imageLoadedEffect,
  imagePreview
};
//# sourceMappingURL=image.js.map
